@@ -13,7 +13,7 @@ function configure_ovs()
     git clone https://github.com/openvswitch/ovs.git ovs_src
     pushd ovs_src
     ./boot.sh && ./configure $* || { cat config.log; exit 1; }
-    make -j4
+    make -j4 || { cat config.log; exit 1; }
     popd
 }
 
@@ -42,16 +42,14 @@ if [ "$TESTSUITE" ]; then
     configure_ovn
 
     export DISTCHECK_CONFIGURE_FLAGS="$OPTS --with-ovs-source=$PWD/ovs_src"
-    if ! make distcheck -j4 TESTSUITEFLAGS="-j4 -k ovn" RECHECK=yes; then
+    if ! make distcheck -j4 TESTSUITEFLAGS="-j4" RECHECK=yes; then
         # testsuite.log is necessary for debugging.
-        cat */_build/tests/testsuite.log
+        cat */_build/sub/tests/testsuite.log
         exit 1
     fi
 else
     configure_ovn $OPTS
-    make selinux-policy
-
-    make -j4
+    make -j4 || { cat config.log; exit 1; }
 fi
 
 exit 0
