@@ -59,6 +59,10 @@ struct condition_aux {
     struct ovsdb_idl_index *sbrec_port_binding_by_name;
     const struct sbrec_chassis *chassis;
     const struct sset *active_tunnels;
+    const struct sbrec_logical_flow *lflow;
+    /* Resource reference to store the port name referenced
+     * in is_chassis_resident() to the logical flow. */
+    struct lflow_resource_ref *lfrr;
 };
 
 static bool
@@ -68,6 +72,8 @@ consider_logical_flow(const struct sbrec_logical_flow *lflow,
                       struct controller_event_options *controller_event_opts,
                       struct lflow_ctx_in *l_ctx_in,
                       struct lflow_ctx_out *l_ctx_out);
+static void lflow_resource_add(struct lflow_resource_ref *, enum ref_type,
+                               const char *ref_name, const struct uuid *);
 
 static bool
 lookup_port_cb(const void *aux_, const char *port_name, unsigned int *portp)
@@ -120,6 +126,14 @@ is_chassis_resident_cb(const void *c_aux_, const char *port_name)
     if (!pb) {
         return false;
     }
+
+    /* Store the port_name to lflow reference. */
+    int64_t dp_id = pb->datapath->tunnel_key;
+    char buf[16];
+    get_unique_lport_key(dp_id, pb->tunnel_key, buf, sizeof(buf));
+    lflow_resource_add(c_aux->lfrr, REF_TYPE_PORTBINDING, buf,
+                       &c_aux->lflow->header_.uuid);
+
     if (strcmp(pb->type, "chassisredirect")) {
         /* for non-chassisredirect ports */
         return pb->chassis && pb->chassis == c_aux->chassis;
@@ -328,31 +342,41 @@ lflow_handle_changed_flows(struct lflow_ctx_in *l_ctx_in,
     struct hmap nd_ra_opts = HMAP_INITIALIZER(&nd_ra_opts);
     nd_ra_opts_init(&nd_ra_opts);
 
+    /* Handle removed flows first, and then other flows, so that when
+     * the flows being added and removed have same match conditions
+     * can be processed in the proper order */
+    SBREC_LOGICAL_FLOW_TABLE_FOR_EACH_TRACKED (lflow,
+                                               l_ctx_in->logical_flow_table) {
+        /* Remove any flows that should be removed. */
+        if (sbrec_logical_flow_is_deleted(lflow)) {
+            VLOG_DBG("handle deleted lflow "UUID_FMT,
+                     UUID_ARGS(&lflow->header_.uuid));
+            ofctrl_remove_flows(l_ctx_out->flow_table, &lflow->header_.uuid);
+            /* Delete entries from lflow resource reference. */
+            lflow_resource_destroy_lflow(l_ctx_out->lfrr,
+                                         &lflow->header_.uuid);
+        }
+    }
+
     struct controller_event_options controller_event_opts;
     controller_event_opts_init(&controller_event_opts);
 
-    /* Flood remove the flows for all the tracked lflows.  Its possible that
-     * lflow_add_flows_for_datapath() may have been called before calling
-     * this function. */
-    struct hmap flood_remove_nodes = HMAP_INITIALIZER(&flood_remove_nodes);
-    struct ofctrl_flood_remove_node *ofrn, *next;
     SBREC_LOGICAL_FLOW_TABLE_FOR_EACH_TRACKED (lflow,
                                                l_ctx_in->logical_flow_table) {
-        VLOG_DBG("delete lflow "UUID_FMT, UUID_ARGS(&lflow->header_.uuid));
-        ofrn = xmalloc(sizeof *ofrn);
-        ofrn->sb_uuid = lflow->header_.uuid;
-        hmap_insert(&flood_remove_nodes, &ofrn->hmap_node,
-                    uuid_hash(&ofrn->sb_uuid));
-    }
-    ofctrl_flood_remove_flows(l_ctx_out->flow_table, &flood_remove_nodes);
-    HMAP_FOR_EACH (ofrn, hmap_node, &flood_remove_nodes) {
-        /* Delete entries from lflow resource reference. */
-        lflow_resource_destroy_lflow(l_ctx_out->lfrr, &ofrn->sb_uuid);
-        /* Reprocessing the lflow if the sb record is not deleted. */
-        lflow = sbrec_logical_flow_table_get_for_uuid(
-            l_ctx_in->logical_flow_table, &ofrn->sb_uuid);
-        if (lflow) {
-            VLOG_DBG("re-add lflow "UUID_FMT,
+        if (!sbrec_logical_flow_is_deleted(lflow)) {
+            /* Now, add/modify existing flows. If the logical
+             * flow is a modification, just remove the flows
+             * for this row, and then add new flows. */
+            if (!sbrec_logical_flow_is_new(lflow)) {
+                VLOG_DBG("handle updated lflow "UUID_FMT,
+                         UUID_ARGS(&lflow->header_.uuid));
+                ofctrl_remove_flows(l_ctx_out->flow_table,
+                                    &lflow->header_.uuid);
+                /* Delete entries from lflow resource reference. */
+                lflow_resource_destroy_lflow(l_ctx_out->lfrr,
+                                             &lflow->header_.uuid);
+            }
+            VLOG_DBG("handle new lflow "UUID_FMT,
                      UUID_ARGS(&lflow->header_.uuid));
             if (!consider_logical_flow(lflow, &dhcp_opts, &dhcpv6_opts,
                                        &nd_ra_opts, &controller_event_opts,
@@ -362,12 +386,6 @@ lflow_handle_changed_flows(struct lflow_ctx_in *l_ctx_in,
             }
         }
     }
-    HMAP_FOR_EACH_SAFE (ofrn, next, hmap_node, &flood_remove_nodes) {
-        hmap_remove(&flood_remove_nodes, &ofrn->hmap_node);
-        free(ofrn);
-    }
-    hmap_destroy(&flood_remove_nodes);
-
     dhcp_opts_destroy(&dhcp_opts);
     dhcp_opts_destroy(&dhcpv6_opts);
     nd_ra_opts_destroy(&nd_ra_opts);
@@ -402,6 +420,7 @@ lflow_handle_changed_ref(enum ref_type ref_type, const char *ref_name,
      * when reparsing the lflows. */
     LIST_FOR_EACH (lrln, ref_list, &rlfn->ref_lflow_head) {
         ovs_list_remove(&lrln->lflow_list);
+        lflow_resource_destroy_lflow(l_ctx_out->lfrr, &lrln->lflow_uuid);
     }
 
     struct hmap dhcp_opts = HMAP_INITIALIZER(&dhcp_opts);
@@ -427,32 +446,22 @@ lflow_handle_changed_ref(enum ref_type ref_type, const char *ref_name,
     controller_event_opts_init(&controller_event_opts);
 
     /* Re-parse the related lflows. */
-    /* Firstly, flood remove the flows from desired flow table. */
-    struct hmap flood_remove_nodes = HMAP_INITIALIZER(&flood_remove_nodes);
-    struct ofctrl_flood_remove_node *ofrn, *ofrn_next;
     LIST_FOR_EACH (lrln, ref_list, &rlfn->ref_lflow_head) {
+        const struct sbrec_logical_flow *lflow =
+            sbrec_logical_flow_table_get_for_uuid(l_ctx_in->logical_flow_table,
+                                                  &lrln->lflow_uuid);
+        if (!lflow) {
+            VLOG_DBG("Reprocess lflow "UUID_FMT" for resource type: %d,"
+                     " name: %s - not found.",
+                     UUID_ARGS(&lrln->lflow_uuid),
+                     ref_type, ref_name);
+            continue;
+        }
         VLOG_DBG("Reprocess lflow "UUID_FMT" for resource type: %d,"
                  " name: %s.",
                  UUID_ARGS(&lrln->lflow_uuid),
                  ref_type, ref_name);
-        ofctrl_flood_remove_add_node(&flood_remove_nodes, &lrln->lflow_uuid);
-    }
-    ofctrl_flood_remove_flows(l_ctx_out->flow_table, &flood_remove_nodes);
-
-    /* Secondly, for each lflow that is actually removed, reprocessing it. */
-    HMAP_FOR_EACH (ofrn, hmap_node, &flood_remove_nodes) {
-        lflow_resource_destroy_lflow(l_ctx_out->lfrr, &ofrn->sb_uuid);
-
-        const struct sbrec_logical_flow *lflow =
-            sbrec_logical_flow_table_get_for_uuid(l_ctx_in->logical_flow_table,
-                                                  &ofrn->sb_uuid);
-        if (!lflow) {
-            VLOG_DBG("lflow "UUID_FMT" not found while reprocessing for"
-                     " resource type: %d, name: %s.",
-                     UUID_ARGS(&ofrn->sb_uuid),
-                     ref_type, ref_name);
-            continue;
-        }
+        ofctrl_remove_flows(l_ctx_out->flow_table, &lrln->lflow_uuid);
 
         if (!consider_logical_flow(lflow, &dhcp_opts, &dhcpv6_opts,
                                    &nd_ra_opts, &controller_event_opts,
@@ -462,11 +471,6 @@ lflow_handle_changed_ref(enum ref_type ref_type, const char *ref_name,
         }
         *changed = true;
     }
-    HMAP_FOR_EACH_SAFE (ofrn, ofrn_next, hmap_node, &flood_remove_nodes) {
-        hmap_remove(&flood_remove_nodes, &ofrn->hmap_node);
-        free(ofrn);
-    }
-    hmap_destroy(&flood_remove_nodes);
 
     LIST_FOR_EACH_SAFE (lrln, next, ref_list, &rlfn->ref_lflow_head) {
         ovs_list_remove(&lrln->ref_list);
@@ -563,7 +567,9 @@ consider_logical_flow(const struct sbrec_logical_flow *lflow,
     struct sset port_groups_ref = SSET_INITIALIZER(&port_groups_ref);
     expr = expr_parse_string(lflow->match, &symtab, l_ctx_in->addr_sets,
                              l_ctx_in->port_groups,
-                             &addr_sets_ref, &port_groups_ref, &error);
+                             &addr_sets_ref, &port_groups_ref,
+                             lflow->logical_datapath->tunnel_key,
+                             &error);
     const char *addr_set_name;
     SSET_FOR_EACH (addr_set_name, &addr_sets_ref) {
         lflow_resource_add(l_ctx_out->lfrr, REF_TYPE_ADDRSET, addr_set_name,
@@ -605,6 +611,8 @@ consider_logical_flow(const struct sbrec_logical_flow *lflow,
         .sbrec_port_binding_by_name = l_ctx_in->sbrec_port_binding_by_name,
         .chassis = l_ctx_in->chassis,
         .active_tunnels = l_ctx_in->active_tunnels,
+        .lflow = lflow,
+        .lfrr = l_ctx_out->lfrr
     };
     expr = expr_simplify(expr, is_chassis_resident_cb, &cond_aux);
     expr = expr_normalize(expr);
@@ -659,7 +667,9 @@ consider_logical_flow(const struct sbrec_logical_flow *lflow,
             if (port_id) {
                 int64_t dp_id = lflow->logical_datapath->tunnel_key;
                 char buf[16];
-                snprintf(buf, sizeof(buf), "%"PRId64"_%"PRId64, dp_id, port_id);
+                get_unique_lport_key(dp_id, port_id, buf, sizeof(buf));
+                lflow_resource_add(l_ctx_out->lfrr, REF_TYPE_PORTBINDING, buf,
+                                   &lflow->header_.uuid);
                 if (!sset_contains(l_ctx_in->local_lport_ids, buf)) {
                     VLOG_DBG("lflow "UUID_FMT
                              " port %s in match is not local, skip",
@@ -772,13 +782,15 @@ consider_neighbor_flow(struct ovsdb_idl_index *sbrec_port_binding_by_name,
 
     uint64_t stub[1024 / 8];
     struct ofpbuf ofpacts = OFPBUF_STUB_INITIALIZER(stub);
+    uint8_t value = 1;
     put_load(mac.ea, sizeof mac.ea, MFF_ETH_DST, 0, 48, &ofpacts);
+    put_load(&value, sizeof value, MFF_LOG_FLAGS, MLF_LOOKUP_MAC_BIT, 1,
+             &ofpacts);
     ofctrl_add_flow(flow_table, OFTABLE_MAC_BINDING, 100,
                     b->header_.uuid.parts[0], &get_arp_match,
                     &ofpacts, &b->header_.uuid);
 
     ofpbuf_clear(&ofpacts);
-    uint8_t value = 1;
     put_load(&value, sizeof value, MFF_LOG_FLAGS, MLF_LOOKUP_MAC_BIT, 1,
              &ofpacts);
     match_set_dl_src(&lookup_arp_match, mac);
@@ -858,3 +870,73 @@ lflow_destroy(void)
     expr_symtab_destroy(&symtab);
     shash_destroy(&symtab);
 }
+
+bool
+lflow_add_flows_for_datapath(const struct sbrec_datapath_binding *dp,
+                             struct lflow_ctx_in *l_ctx_in,
+                             struct lflow_ctx_out *l_ctx_out)
+{
+    bool handled = true;
+    struct hmap dhcp_opts = HMAP_INITIALIZER(&dhcp_opts);
+    struct hmap dhcpv6_opts = HMAP_INITIALIZER(&dhcpv6_opts);
+    const struct sbrec_dhcp_options *dhcp_opt_row;
+    SBREC_DHCP_OPTIONS_TABLE_FOR_EACH (dhcp_opt_row,
+                                       l_ctx_in->dhcp_options_table) {
+        dhcp_opt_add(&dhcp_opts, dhcp_opt_row->name, dhcp_opt_row->code,
+                     dhcp_opt_row->type);
+    }
+
+
+    const struct sbrec_dhcpv6_options *dhcpv6_opt_row;
+    SBREC_DHCPV6_OPTIONS_TABLE_FOR_EACH (dhcpv6_opt_row,
+                                         l_ctx_in->dhcpv6_options_table) {
+       dhcp_opt_add(&dhcpv6_opts, dhcpv6_opt_row->name, dhcpv6_opt_row->code,
+                    dhcpv6_opt_row->type);
+    }
+
+    struct hmap nd_ra_opts = HMAP_INITIALIZER(&nd_ra_opts);
+    nd_ra_opts_init(&nd_ra_opts);
+
+    struct controller_event_options controller_event_opts;
+    controller_event_opts_init(&controller_event_opts);
+
+    struct sbrec_logical_flow *lf_row = sbrec_logical_flow_index_init_row(
+        l_ctx_in->sbrec_logical_flow_by_logical_datapath);
+    sbrec_logical_flow_index_set_logical_datapath(lf_row, dp);
+
+    const struct sbrec_logical_flow *lflow;
+    SBREC_LOGICAL_FLOW_FOR_EACH_EQUAL (
+        lflow, lf_row, l_ctx_in->sbrec_logical_flow_by_logical_datapath) {
+        /* Remove the lflow from flow_table if present before processing it. */
+        ofctrl_remove_flows(l_ctx_out->flow_table, &lflow->header_.uuid);
+
+        if (!consider_logical_flow(lflow, &dhcp_opts, &dhcpv6_opts,
+                                   &nd_ra_opts, &controller_event_opts,
+                                   l_ctx_in, l_ctx_out)) {
+            handled = false;
+            break;
+        }
+    }
+    sbrec_logical_flow_index_destroy_row(lf_row);
+
+    dhcp_opts_destroy(&dhcp_opts);
+    dhcp_opts_destroy(&dhcpv6_opts);
+    nd_ra_opts_destroy(&nd_ra_opts);
+    controller_event_opts_destroy(&controller_event_opts);
+    return handled;
+}
+
+bool
+lflow_handle_flows_for_lport(const struct sbrec_port_binding *pb,
+                             struct lflow_ctx_in *l_ctx_in,
+                             struct lflow_ctx_out *l_ctx_out)
+{
+    bool changed;
+    int64_t dp_id = pb->datapath->tunnel_key;
+    char pb_ref_name[16];
+    get_unique_lport_key(dp_id, pb->tunnel_key, pb_ref_name,
+                         sizeof(pb_ref_name));
+
+    return lflow_handle_changed_ref(REF_TYPE_PORTBINDING, pb_ref_name,
+                                    l_ctx_in, l_ctx_out, &changed);
+}
