@@ -391,12 +391,14 @@ struct expr *expr_parse(struct lexer *, const struct shash *symtab,
                         const struct shash *addr_sets,
                         const struct shash *port_groups,
                         struct sset *addr_sets_ref,
-                        struct sset *port_groups_ref);
+                        struct sset *port_groups_ref,
+                        int64_t dp_id);
 struct expr *expr_parse_string(const char *, const struct shash *symtab,
                                const struct shash *addr_sets,
                                const struct shash *port_groups,
                                struct sset *addr_sets_ref,
                                struct sset *port_groups_ref,
+                               int64_t dp_id,
                                char **errorp);
 
 struct expr *expr_clone(struct expr *);
