@@ -39,6 +39,7 @@
 #include "openvswitch/shash.h"
 #include "simap.h"
 #include "util.h"
+#include "controller/lflow.h"
 
 /* --relops: Bitmap of the relational operators to test, in exhaustive test. */
 static unsigned int test_relops;
@@ -187,6 +188,8 @@ create_gen_opts(struct hmap *dhcp_opts, struct hmap *dhcpv6_opts,
     dhcp_opt_add(dhcp_opts, "bootfile_name", 67, "str");
     dhcp_opt_add(dhcp_opts, "path_prefix", 210, "str");
     dhcp_opt_add(dhcp_opts, "tftp_server_address", 150, "ipv4");
+    dhcp_opt_add(dhcp_opts, "arp_cache_timeout", 35, "uint32");
+    dhcp_opt_add(dhcp_opts, "tcp_keepalive_interval", 38, "uint32");
 
     /* DHCPv6 options. */
     hmap_init(dhcpv6_opts);
@@ -235,8 +238,8 @@ create_port_groups(struct shash *port_groups)
     };
     static const char *const pg2[] = { NULL };
 
-    expr_const_sets_add(port_groups, "pg1", pg1, 3, false);
-    expr_const_sets_add(port_groups, "pg_empty", pg2, 0, false);
+    expr_const_sets_add(port_groups, "0_pg1", pg1, 3, false);
+    expr_const_sets_add(port_groups, "0_pg_empty", pg2, 0, false);
 }
 
 static bool
@@ -302,7 +305,7 @@ test_parse_expr__(int steps)
         char *error;
 
         expr = expr_parse_string(ds_cstr(&input), &symtab, &addr_sets,
-                                 &port_groups, NULL, NULL, &error);
+                                 &port_groups, NULL, NULL, 0, &error);
         if (!error && steps > 0) {
             expr = expr_annotate(expr, &symtab, &error);
         }
@@ -428,7 +431,7 @@ test_evaluate_expr(struct ovs_cmdl_context *ctx)
         struct expr *expr;
 
         expr = expr_parse_string(ds_cstr(&input), &symtab, NULL, NULL,
-                                 NULL, NULL, &error);
+                                 NULL, NULL, 0, &error);
         if (!error) {
             expr = expr_annotate(expr, &symtab, &error);
         }
@@ -903,7 +906,7 @@ test_tree_shape_exhaustively(struct expr *expr, struct shash *symtab,
 
             char *error;
             modified = expr_parse_string(ds_cstr(&s), symtab, NULL,
-                                         NULL, NULL, NULL, &error);
+                                         NULL, NULL, NULL, 0, &error);
             if (error) {
                 fprintf(stderr, "%s fails to parse (%s)\n",
                         ds_cstr(&s), error);
@@ -1330,11 +1333,11 @@ test_parse_actions(struct ovs_cmdl_context *ctx OVS_UNUSED)
                 .meter_table = &meter_table,
 
                 .pipeline = OVNACT_P_INGRESS,
-                .ingress_ptable = 8,
-                .egress_ptable = 40,
-                .output_ptable = 64,
-                .mac_bind_ptable = 65,
-                .mac_lookup_ptable = 67,
+                .ingress_ptable = OFTABLE_LOG_INGRESS_PIPELINE,
+                .egress_ptable = OFTABLE_LOG_EGRESS_PIPELINE,
+                .output_ptable = OFTABLE_SAVE_INPORT,
+                .mac_bind_ptable = OFTABLE_MAC_BINDING,
+                .mac_lookup_ptable = OFTABLE_MAC_LOOKUP,
             };
             struct ofpbuf ofpacts;
             ofpbuf_init(&ofpacts, 0);
