@@ -239,7 +239,7 @@ parse_options(int argc, char *argv[])
         OPT_CT,
         OPT_FRIENDLY_NAMES,
         OPT_NO_FRIENDLY_NAMES,
-        DAEMON_OPTION_ENUMS,
+        OVN_DAEMON_OPTION_ENUMS,
         SSL_OPTION_ENUMS,
         VLOG_OPTION_ENUMS,
         OPT_LB_DST,
@@ -260,7 +260,7 @@ parse_options(int argc, char *argv[])
         {"version", no_argument, NULL, 'V'},
         {"lb-dst", required_argument, NULL, OPT_LB_DST},
         {"select-id", required_argument, NULL, OPT_SELECT_ID},
-        DAEMON_LONG_OPTIONS,
+        OVN_DAEMON_LONG_OPTIONS,
         VLOG_LONG_OPTIONS,
         STREAM_SSL_LONG_OPTIONS,
         {NULL, 0, NULL, 0},
@@ -333,7 +333,7 @@ parse_options(int argc, char *argv[])
             printf("DB Schema %s\n", sbrec_get_db_version());
             exit(EXIT_SUCCESS);
 
-        DAEMON_OPTION_HANDLERS
+        OVN_DAEMON_OPTION_HANDLERS
         VLOG_OPTION_HANDLERS
         STREAM_SSL_OPTION_HANDLERS
 
@@ -476,7 +476,6 @@ static struct shash port_groups;
 static struct hmap dhcp_opts;   /* Contains "struct gen_opts_map"s. */
 static struct hmap dhcpv6_opts; /* Contains "struct gen_opts_map"s. */
 static struct hmap nd_ra_opts; /* Contains "struct gen_opts_map"s. */
-static struct controller_event_options event_opts;
 
 static struct ovntrace_datapath *
 ovntrace_datapath_find_by_sb_uuid(const struct uuid *sb_uuid)
@@ -583,13 +582,13 @@ ovntrace_mac_binding_find(const struct ovntrace_datapath *dp,
 static const struct ovntrace_mac_binding *
 ovntrace_mac_binding_find_mac_ip(const struct ovntrace_datapath *dp,
                                  uint16_t port_key, const struct in6_addr *ip,
-                                 struct eth_addr mac)
+                                 struct eth_addr *mac)
 {
     const struct ovntrace_mac_binding *bind;
     HMAP_FOR_EACH_WITH_HASH (bind, node, hash_mac_binding(port_key, ip),
                              &dp->mac_bindings) {
         if (bind->port_key == port_key && ipv6_addr_equals(ip, &bind->ip)
-            && eth_addr_equals(bind->mac, mac)) {
+            && (!mac || eth_addr_equals(bind->mac, *mac))) {
             return bind;
         }
     }
@@ -890,7 +889,8 @@ read_flows(void)
         char *error;
         struct expr *match;
         match = expr_parse_string(sblf->match, &symtab, &address_sets,
-                                  &port_groups, NULL, NULL, &error);
+                                  &port_groups, NULL, NULL, dp->tunnel_key,
+                                  &error);
         if (error) {
             VLOG_WARN("%s: parsing expression failed (%s)",
                       sblf->match, error);
@@ -903,7 +903,6 @@ read_flows(void)
             .dhcp_opts = &dhcp_opts,
             .dhcpv6_opts = &dhcpv6_opts,
             .nd_ra_opts = &nd_ra_opts,
-            .controller_event_opts = &event_opts,
             .pipeline = (!strcmp(sblf->pipeline, "ingress")
                          ? OVNACT_P_INGRESS
                          : OVNACT_P_EGRESS),
@@ -983,8 +982,6 @@ read_gen_opts(void)
 
     hmap_init(&nd_ra_opts);
     nd_ra_opts_init(&nd_ra_opts);
-
-    controller_event_opts_init(&event_opts);
 }
 
 static void
@@ -1749,10 +1746,10 @@ execute_get_mac_bind(const struct ovnact_get_mac_bind *bind,
 }
 
 static void
-execute_lookup_mac(const struct ovnact_lookup_mac_bind *bind OVS_UNUSED,
-                   const struct ovntrace_datapath *dp OVS_UNUSED,
-                   struct flow *uflow OVS_UNUSED,
-                   struct ovs_list *super OVS_UNUSED)
+execute_lookup_mac_bind(const struct ovnact_lookup_mac_bind *bind,
+                        const struct ovntrace_datapath *dp,
+                        struct flow *uflow,
+                        struct ovs_list *super)
 {
     /* Get logical port number.*/
     struct mf_subfield port_sf = expr_resolve_field(&bind->port);
@@ -1775,7 +1772,7 @@ execute_lookup_mac(const struct ovnact_lookup_mac_bind *bind OVS_UNUSED,
     mf_read_subfield(&mac_sf, uflow, &mac_sv);
 
     const struct ovntrace_mac_binding *binding
-        = ovntrace_mac_binding_find_mac_ip(dp, port_key, &ip, mac_sv.mac);
+        = ovntrace_mac_binding_find_mac_ip(dp, port_key, &ip, &mac_sv.mac);
 
     struct mf_subfield dst = expr_resolve_field(&bind->dst);
     uint8_t val = 0;
@@ -1793,6 +1790,44 @@ execute_lookup_mac(const struct ovnact_lookup_mac_bind *bind OVS_UNUSED,
     mf_write_subfield_flow(&dst, &sv, uflow);
 }
 
+static void
+execute_lookup_mac_bind_ip(const struct ovnact_lookup_mac_bind_ip *bind,
+                           const struct ovntrace_datapath *dp,
+                           struct flow *uflow,
+                           struct ovs_list *super)
+{
+    /* Get logical port number.*/
+    struct mf_subfield port_sf = expr_resolve_field(&bind->port);
+    ovs_assert(port_sf.n_bits == 32);
+    uint32_t port_key = mf_get_subfield(&port_sf, uflow);
+
+    /* Get IP address. */
+    struct mf_subfield ip_sf = expr_resolve_field(&bind->ip);
+    ovs_assert(ip_sf.n_bits == 32 || ip_sf.n_bits == 128);
+    union mf_subvalue ip_sv;
+    mf_read_subfield(&ip_sf, uflow, &ip_sv);
+    struct in6_addr ip = (ip_sf.n_bits == 32
+                          ? in6_addr_mapped_ipv4(ip_sv.ipv4)
+                          : ip_sv.ipv6);
+
+    const struct ovntrace_mac_binding *binding
+        = ovntrace_mac_binding_find_mac_ip(dp, port_key, &ip, NULL);
+
+    struct mf_subfield dst = expr_resolve_field(&bind->dst);
+    uint8_t val = 0;
+
+    if (binding) {
+        val = 1;
+        ovntrace_node_append(super, OVNTRACE_NODE_ACTION,
+                             "/* MAC binding found. */");
+    } else {
+        ovntrace_node_append(super, OVNTRACE_NODE_ACTION,
+                             "/* lookup failed - No MAC binding. */");
+    }
+    union mf_subvalue sv = { .u8_val = val };
+    mf_write_subfield_flow(&dst, &sv, uflow);
+}
+
 static void
 execute_put_opts(const struct ovnact_put_opts *po,
                  const char *name, struct flow *uflow,
@@ -2212,11 +2247,22 @@ trace_actions(const struct ovnact *ovnacts, size_t ovnacts_len,
             break;
 
         case OVNACT_LOOKUP_ARP:
-            execute_lookup_mac(ovnact_get_LOOKUP_ARP(a), dp, uflow, super);
+            execute_lookup_mac_bind(ovnact_get_LOOKUP_ARP(a), dp, uflow,
+                                    super);
             break;
 
         case OVNACT_LOOKUP_ND:
-            execute_lookup_mac(ovnact_get_LOOKUP_ND(a), dp, uflow, super);
+            execute_lookup_mac_bind(ovnact_get_LOOKUP_ND(a), dp, uflow, super);
+            break;
+
+        case OVNACT_LOOKUP_ARP_IP:
+            execute_lookup_mac_bind_ip(ovnact_get_LOOKUP_ARP_IP(a), dp, uflow,
+                                       super);
+            break;
+
+        case OVNACT_LOOKUP_ND_IP:
+            execute_lookup_mac_bind_ip(ovnact_get_LOOKUP_ND_IP(a), dp, uflow,
+                                       super);
             break;
 
         case OVNACT_PUT_DHCPV4_OPTS:
@@ -2300,6 +2346,8 @@ trace_actions(const struct ovnact *ovnacts, size_t ovnacts_len,
 
         case OVNACT_FWD_GROUP:
             break;
+        case OVNACT_DHCP6_REPLY:
+            break;
         }
     }
     ds_destroy(&s);
@@ -2328,7 +2376,7 @@ trace_openflow(const struct ovntrace_flow *f, struct ovs_list *super)
 
     struct ofputil_flow_stats *fses;
     size_t n_fses;
-    int error = vconn_dump_flows(vconn, &fsr, OFPUTIL_P_OF13_OXM,
+    int error = vconn_dump_flows(vconn, &fsr, OFPUTIL_P_OF15_OXM,
                                  &fses, &n_fses);
     if (error) {
         ovntrace_node_append(super, OVNTRACE_NODE_ERROR,
@@ -2437,7 +2485,7 @@ trace(const char *dp_s, const char *flow_s)
     ds_put_char(&output, '\n');
 
     if (ovs) {
-        int retval = vconn_open_block(ovs, 1 << OFP13_VERSION, 0, -1, &vconn);
+        int retval = vconn_open_block(ovs, 1 << OFP15_VERSION, 0, -1, &vconn);
         if (retval) {
             VLOG_WARN("%s: connection failed (%s)", ovs, ovs_strerror(retval));
         }
